/*
 *  ircd-hybrid: an advanced Internet Relay Chat Daemon(ircd).
 *  m_svsmode.c: Sets a user or channel mode.
 *
 *  Copyright (C) 2002 by the past and present ircd coders, and others.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 *  USA
 *
 *  $Id: m_svsmode.c,v 1.8 2002/08/27 22:04:00 strtok Exp $
 */

#include "stdinc.h"
#include "tools.h"
#include "handlers.h"
#include "channel.h"
#include "channel_mode.h"
#include "vchannel.h"
#include "client.h"
#include "hash.h"
#include "irc_string.h"
#include "ircd.h"
#include "numeric.h"
#include "s_user.h"
#include "s_conf.h"
#include "s_serv.h"
#include "send.h"
#include "msg.h"
#include "parse.h"
#include "modules.h"
#include "packet.h"
#include "common.h"

extern BlockHeap *ban_heap;

static void remove_matching_bans(struct Client *,struct Client *, struct Channel *, 
                                 struct Client *);
static void channel_svsmode(struct Client *, struct Client *, int , char **);
static void m_svsmode(struct Client*, struct Client*, int, char**);

struct Message mode_msgtab = {
  "SVSMODE", 0, 0, 2, 0, MFLG_SLOW, 0,
  {m_unregistered, m_svsmode, m_svsmode, m_svsmode}
};
#ifndef STATIC_MODULES

void
_modinit(void)
{
  mod_add_cmd(&mode_msgtab);
}

void
_moddeinit(void)
{
  mod_del_cmd(&mode_msgtab);
}


const char *_version = "$Revision: 1.8 $";
#endif
/*
 * m_svsmode - SVSMODE command handler
 * parv[0] - sender
 * parv[1] - channel
 * parv[2] - modes
 * parv[3] - nick (depending on modes)
 * parv[4] - nickts (depending on modes)
 *
 * parv[0] - sender
 * parv[1] - user
 * parv[2] - modes
 */
static void m_svsmode(struct Client *client_p, struct Client *source_p,
              int parc, char *parv[])
{

  struct Client *target_p;
  int badflag = NO;
  int what, svs_stamp;
  char **p, *m;

  what = MODE_ADD;
  
  /* Return quietly if this
   * is a non service 
   */

  if(!IsService(source_p))
     return; 

  if (parc < 3)
  {
      sendto_one(source_p, form_str(ERR_NEEDMOREPARAMS),
                 me.name, parv[0], "SVSMODE");
      return;
  }

  if (parv[1][0] == '#')
  {
     channel_svsmode(client_p, source_p, parc, parv);
     return;
  }

  if ((target_p = find_person(parv[1])) == NULL)
  {
      if (MyConnect(source_p))
        sendto_one(source_p, form_str(ERR_NOSUCHCHANNEL),
                   me.name, parv[0], parv[1]);
      return;
  }

  for(p = &parv[2]; p && *p; p++)
     for(m = *p; *m; m++)
        switch(*m)
        {
           case '+':
              what = MODE_ADD;
              break;
           case '-':
              what = MODE_DEL;
               break;
           case 'r':
           case 'd':
              if(what == MODE_ADD)
              {
                 if(parc >= 4)
                 {
                    svs_stamp = atoi(parv[3]);
                    target_p->user->svs_stamp = svs_stamp;
                 }
                 else
                    target_p->umodes |= FLAGS_IDENTIFIED;
                   
              }
              else
              {
                 target_p->umodes &= ~FLAGS_IDENTIFIED;
              }
              break;
        }
  /* Propogate the SVSMODE to other servers */
  sendto_server(client_p, source_p, NULL, NOCAPS, NOCAPS, NOFLAGS,
                 ":%s SVSMODE %s %s %s", source_p->name, target_p->name,
                  parv[2], (parc >= 4) ? parv[3] : "");
}


/*  channel_svsmode
 *
 * parv[0] - sender
 * parv[1] - channel
 * parv[2] - modes
 * parv[3] - nick (depending on modes)
 * parv[4] - nickts (depending on modes)
 *
 */

static void channel_svsmode(struct Client *client_p, struct Client *source_p,
              int parc, char *parv[])
{
   struct Client *target_p;
   struct Channel *chptr;
   char **p, *m;
   int what;

   what = MODE_ADD;

   /* Right now all we do in channel_svsmode is -b, and that needs
      at least 4 parameters. */

   if (parc < 4)
   {
       sendto_one(source_p, form_str(ERR_NEEDMOREPARAMS),
                 me.name, parv[0], "SVSMODE");
       return;
   }


   if((chptr = hash_find_channel(parv[1])) == NULL)
   {
       sendto_one(source_p, form_str(ERR_NOSUCHCHANNEL),
                     me.name, parv[0], parv[1]);
       return;
   }

   if((target_p = find_client(parv[3])) == NULL)
   {
      sendto_one(source_p, form_str(RPL_ENDOFWHOIS), me.name, 
                 parv[0], parv[3]);
      return;
   }
   
  for(p = &parv[2]; p && *p; p++)
     for(m = *p; *m; m++)
        switch(*m)
        {
           case '+':
              what = MODE_ADD;
              break;
           case '-':
              what = MODE_DEL;
              break;
           case 'b':
              if(what == MODE_DEL)
                 remove_matching_bans(client_p, source_p, chptr, target_p);
              break;
           default:
              break;
        }

  /* Propogate the SVSMODE to other servers */
  sendto_server(client_p, source_p, NULL, NOCAPS, NOCAPS, NOFLAGS,
                 ":%s SVSMODE %s %s %s", source_p->name, target_p->name,
                  parv[2], (parc >= 4) ? parv[3] : "");

}

/*
 *
 *
 */

static void remove_matching_bans(struct Client *client_p, struct Client *source_p, 
                                 struct Channel *chptr, struct Client *target_p)
{

   dlink_node *ban, *ban_next;
   struct Ban *actualBan = NULL;

   char target_host[NICKLEN + USERLEN + HOSTLEN + 6];
   char target_iphost[NICKLEN + USERLEN + HOSTLEN + 6];

   char modebuf[] = "-b";
   char parabuf[MODEBUFLEN];

   char *parv[2];

  /* We can only perform remove_matching_bans on a local client
     because we need to check against IP too */
  if(!MyClient(target_p))
      return;


  ircsprintf(target_host,"%s!%s@%s", target_p->name, target_p->username, 
             target_p->host);
  ircsprintf(target_iphost,"%s!%s@%s", target_p->name, target_p->username,
             target_p->localClient->sockhost);

  DLINK_FOREACH_SAFE(ban, ban_next, chptr->banlist.head)
  {

    actualBan = ban->data;

   if (match(actualBan->banstr, target_host) || 
      match(actualBan->banstr, target_iphost))
    {
      /* We form the parv for set_channel_mode which is called
         normally by m_mode. This prevents any duplication of
         hyb7 code (ie. won't break LL, anonops, etc). 

         We copy the ban into another buffer because set_channel_mode
         will delete the ban (thus deleting banstr) before it is
         finished propogating. 

                           (+blitzed)
      */

      strlcpy(parabuf, actualBan->banstr, MODEBUFLEN);

      parv[0] = modebuf;
      parv[1] = parabuf;

      set_channel_mode(client_p, source_p, chptr, 2, parv,
                     chptr->chname);
   }
  }
}
